#pragma once

// Comment this line to avoid debug output
//#define KNX_OUTPUT

#define BOOST_MPL_CFG_NO_PREPROCESSED_HEADERS
#define BOOST_MPL_LIMIT_VECTOR_SIZE 30

#include <string>

#ifdef __linux__
//#define MARK_UNUSED __attribute((unused))
#define MARK_UNUSED
#define TYPENAME typename
#else
#define MARK_UNUSED
#define TYPENAME
#endif

#ifndef __linux__
#ifndef __APPLE__
#pragma warning (disable: 4251)
#endif
#endif

namespace knx
{

//! reset
static const std::string COLOR_OFF="\033[0m"      ; // Text Reset

//! regular
static const std::string Red="\033[0;31m"         ; // Red
static const std::string Green="\033[0;32m"       ; // Green
static const std::string Yellow="\033[0;33m"      ; // Yellow

//! bold
static const std::string BYellow="\033[1;33m"     ; // Yellow
static const std::string BBlue="\033[1;34m"       ; // Blue

//! define colors
static const std::string COLOR_RESET = COLOR_OFF;
static const std::string COLOR_INFO = Green;
static const std::string COLOR_WARNING = Yellow;
static const std::string COLOR_ERROR = Red;
static const std::string COLOR_SENDING = BBlue;
static const std::string COLOR_RECEIVING = BYellow;

} // end namespace

//! cut out the last filename
#define SIMPLE_FILENAME(x) x.substr(x.find_last_of('/')+1)

//! setting if statement
#ifdef KNX_OUTPUT
#define ONOFF
#else
#define ONOFF if(false)
#endif

//! colored output for sending and receiving
#include <iostream>
#define KNX_SENDING ONOFF std::cerr << knx::COLOR_SENDING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET
#define KNX_RECEIVING ONOFF std::cerr << knx::COLOR_RECEIVING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET

//! colored output for info, warn and error messages
#define KNX_INFORMATION ONOFF std::cerr << knx::COLOR_INFO << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") >> " << knx::COLOR_RESET
#define KNX_WARNING ONOFF std::cerr << knx::COLOR_WARNING << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") WARNING >> " << knx::COLOR_RESET
#define KNX_ERROR std::cerr << knx::COLOR_ERROR << SIMPLE_FILENAME(std::string(__FILE__)) << ":" << __LINE__ << " (" << __FUNCTION__ << ") ERROR >> " << knx::COLOR_RESET
#define KNX_FINAL std::cout

// WINDOWS
#ifdef __linux
#define KNX_IMPORT_EXPORT
#elif defined __APPLE__
#define KNX_IMPORT_EXPORT
#elif defined KNX_EXPORT
#define KNX_IMPORT_EXPORT __declspec(dllexport)
#else
#define KNX_IMPORT_EXPORT __declspec(dllimport)
#endif
