#pragma once

#include "debug.hpp"

namespace knx {

//! Base class for dpt 2.
class KNX_IMPORT_EXPORT dpt_2 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & time);
        //! Constructor with control and value.
        data(bool c, bool v);
        //! Standard Constructor setting all to zero.
        data();
        //! Control bit 
        bool control;
        //! Value bit 
        bool value;
    };
};

//! Base class for dpt 3.
class dpt_3 {
public:
    //! Data container for this dpt.
    class data {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & time);
        //! Standard constructor with values.
        KNX_IMPORT_EXPORT data(bool c, uint8_t v);
        //! Standard constructor.
        KNX_IMPORT_EXPORT data();
        //! Control bit 
        bool control;
        //! Control field
        uint8_t value;
    };
};

//! Base class container.
class dpt_10 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & time);
        //! Comparison for time objects.
        bool operator== ( const data & other );
        //! @brief creates a current time value
        data();
        //! day of the week monday (1) till sunday (7)
        uint8_t day;
        //! From 0 to 23.
        uint8_t hour;
        //! From 0 to 59.
        uint8_t minutes;
        //! From 0 to 59.
        uint8_t seconds;
        //! No day assigned or unknown.
        static const uint8_t NODAY = 0;
        //! Monday
        static const uint8_t MONDAY = 1;
        //! Tuesday
        static const uint8_t TUESDAY = 2;
        //! Wednesday
        static const uint8_t WEDNESDAY = 3;
        //! Thursday
        static const uint8_t THURSDAY = 4;
        //! Friday
        static const uint8_t FRIDAY = 5;
        //! Saturday
        static const uint8_t SATURDAY = 6;
        //! Sunday
        static const uint8_t SUNDAY = 7;
    };
};

//! Base class container.
class dpt_11 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & date);
        //! Compare the date.
        bool operator== ( const data & other );
        //! @brief creates a current date value
        data();
        //! Day of month.
        uint8_t day;
        //! Month of year.
        uint8_t month;
        //! Year with two digits. >=90 is expected to start with 19XX.
        uint8_t year;
    };
};

//! Base class container.
class dpt_15 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare access data.
        bool operator== ( const data & other );
        //! Default constructor setting to zero.
        data();
        //! The data storage.
        struct {
            //! digit6
            unsigned int d6 : 4;
            //! digit5
            unsigned int d5 : 4;
            //! digit4
            unsigned int d4 : 4;
            //! digit3
            unsigned int d3 : 4;
            //! digit2
            unsigned int d2 : 4;
            //! digit1
            unsigned int d1 : 4;
            //! Error detection (0 is no error).
            bool e : 1;
            //! Permission (1 is accepted).
            bool p : 1;
            //! Encryption (1 is yes).
            bool c : 1;
            //! Read direction (0 is left to right).
            bool d : 1;
            //! Index of access identification code.
            unsigned int index : 4;
        } storage;
    };
};

//! Base class container.
class dpt_16 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare it
        bool operator== ( const data & other );
        //! Setting all to zero.
        data();
        //! Storage.
        struct {
            //! The string value.
            char ascii[14];
        } storage;
    };
};

//! Base class container.
class dpt_17 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            //! Reserved do not use!.
            unsigned int r : 2;
            //! Scene number with 6 bits.
            unsigned int number : 6;
        } storage;
    };
};

//! Base class container.
class dpt_18 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            //! 0 means activate and 1 learn the scene
            unsigned int c : 1;
            //! do not use
            unsigned int r : 1;
            //! scene nuber
            unsigned int number : 6;
        } storage;
    };
};

//! Base class container.
class dpt_19 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            //! Add 1900 to get real year.
            unsigned int year : 8;

            //! do not use
            unsigned int res0 : 4;
            //! January is 1, February 2, ...
            unsigned int month : 4;

            //! do not use
            unsigned int res1 : 3;
            //! From 1st to 31st day
            unsigned int day_of_month : 5;

            //! Monday is 1, Tuesday is 2, ..
            unsigned int day_of_week : 3;
            //! As usual.
            unsigned int hour_of_day : 5;

            //! do not use
            unsigned int res2 : 2;
            //! Minutes till 59.
            unsigned int minutes : 6;

            //! do not use
            unsigned int res3 : 2;
            //! Seconds till 59.
            unsigned int seconds : 6;

            //! Fault with 1 is true
            unsigned int f : 1;
            //! Set to 1 if it is a working day
            unsigned int wd : 1;
            //! Indicates if wd field is valid. 1 means not valid.
            unsigned int nwd : 1;
            //! Set to one if year is not valid.
            unsigned int ny : 1;
            //! Set to one if day and month are not valid.
            unsigned int nd : 1;
            //! Set to one if day of week is not valid.
            unsigned int ndow : 1;
            //! Set to one if time is not valid.
            unsigned int nt : 1;
            //! Summer time field. Add one hour if set to one.
            unsigned int suti : 1;

            //! Quality of clock. 1 says with external signal.
            unsigned int clq : 1;
            //! do not use
            unsigned int res4 : 7;
        } storage;
    };
};

//! Base class container.
class dpt_26 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            //! Do not use.
            unsigned int reserved : 1;
            //! 1 says scene is inactive.
            unsigned int active : 1;
            //! The number of the scene.
            unsigned int scene_number : 6;
        } storage;
    };
};

//! Base class container.
class dpt_219 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            //! Number of the current log.
            unsigned char log_number;
            //! Priority with 0 is highest
            unsigned char alarm_priority;
            //! Area of error (there is a table where 0 is no fault and 20 is lighting (for example)
            unsigned char application_area;
            //! Class of error. 0 is no fault. 4 is hw fault (table exists)
            unsigned char error_class;
            //! Do not use.
            unsigned int reserverd0 : 2;
            //! Bit field with 0 (ack), 1 (timestamp), 2 (alarmtext), 3 (errorcode) supressed
            unsigned int attributes : 6;
            //! Do not use
            unsigned int reserverd1 : 5;
            //! Alarm status attributes with 0 (inalarm), 1 (alarmunack) and 2 (locked)
            unsigned int alarm_status_attributes : 3;
        } storage;
    };
};

//! Base class container.
class dpt_232 {
public:
    //! Data container for this dpt.
    class KNX_IMPORT_EXPORT data
    {
    public:
        //! Output formatting.
        KNX_IMPORT_EXPORT friend std::ostream & operator << (std::ostream & os, const data & data);
        //! Compare.
        bool operator== ( const data & other );
        //! Set to zero.
        data();
        //! Data storage.
        struct {
            // red color from 0 to 255
            unsigned char red;
            // green color from 0 to 255
            unsigned char green;
            // blue color from 0 to 255
            unsigned char blue;
        } storage;
    };
};

}
